import iris
import numpy as np
import iris.quickplot as qplt
import matplotlib.pyplot as plt
import copy

def nearest_ind(data,val):
    '''Returns the index of the data array which has the value closest to val'''
    ind = abs(data-val).argmin() 
    return ind
    
def nearest_ji(lat,lon,pos):
    '''Returns the (j,i) tuple of the grid point closest to the lat and lon given
    
    Input:
    lat, lon - 2D arrays of lat and lon
    pos - tuple of (lat0,lon0) to find
    
    Output:
    (j,i)
    '''
    lat0,lon0 = tuple(pos)
    if lon0 < lon.min():
       lon0+=360.0
    if lon0 > lon.max():
       lon0-=360.0
    weight = (lat - lat0)**2 + (lon - lon0)**2
    tmp = np.where(weight == weight.min())
    j,i = tmp[0][0],tmp[1][0]
    return (j,i)
    

def haversine(lat1, lon1, lat2, lon2):
    '''Calculate distance between two points given their latitude and longitude'''
    R = 6372.8 #km
    dLat = np.radians(lat2 - lat1)
    dLon = np.radians(lon2 - lon1)
    lat1 = np.radians(lat1)
    lat2 = np.radians(lat2)
    a = np.sin(dLat/2)**2 + np.cos(lat1)*np.cos(lat2)*np.sin(dLon/2)**2
    c = 2*np.arcsin(np.sqrt(a))
    return R * c 
      
def grl_mask(lat,lon,landseamask):
    jsea,isea=np.where(landseamask==1.0)
    jland,iland=np.where(landseamask==0.0)
    dist=copy.deepcopy(landseamask)
    dist[np.where(dist==2.0)]=301.
    for j,i in zip(jsea,isea):
       dist_to_land = haversine(lat[j,i],lon[j,i],lat[jland,iland],lon[jland,iland])
       dist[j,i]=min(dist_to_land.min(),301.)
    lmax=300.    
    region=copy.deepcopy(landseamask)*0.0
    j,i=np.where((dist<lmax) & (dist>0.0))
    region[j,i]=np.exp(-dist[j,i]/lmax)
    return region    

def get_land_sea_mask(lat,lon,tmaskutil):
    # This may need to be modified for different grids.
    mask = tmaskutil[:]*1.0 # land = 0; sea = 1.0; ignore=2.0
    j,i=nearest_ji(lat,lon,(82.0,7.))
    mask[:,i:]=2.0 # ignore regions to east 
    j,i=nearest_ji(lat,lon,(67.0,-61.))
    mask[:,:i]=2.0 # ignore regions to west 
    j,i=nearest_ji(lat,lon,(56.0,-45.))
    mask[:j,:]=2.0 # ignore regions to south 
    j,i=nearest_ji(lat,lon,(67.0,-25.))
    mask[:j,i:]=2.0 # ignore Iceland
    j,i=nearest_ji(lat,lon,(81.5,-13.))
    mask[j:,:]=2.0 # ignore east section of north Greenland
    j,i=nearest_ji(lat,lon,(76.,-68.))
    j2,i2=nearest_ji(lat,lon,(76.,-60.))
    mask[j:,:i2]=2.0 # ignore west section of north Greenland
    return mask

def make_hosing_nc(hosing_rate,fileout):
    '''
    Create a file to input fresh water hosing for an ORCA025 grid
    
    Inputs:
       hosing_rate:   float of hosing rate in Sv
       fileout:       string of file name for output file
       
    Saves resulting field (in kg/m2/s of freshwater) to fileout
    '''
    #Read in data and grid info    
    mesh_file = 'mesh_mask_HighresMIP_eORCA1_extr.nc'
    tmaskutil = iris.load(mesh_file,'tmaskutil')[0][0].data # land mask
    lat = iris.load(mesh_file,'gphit')[0][0].data
    lon = iris.load(mesh_file,'glamt')[0][0].data
    dx = iris.load(mesh_file,'e1t')[0][0].data
    dy = iris.load(mesh_file,'e2t')[0][0].data
    #
    # Make hosing array
    landseamask= get_land_sea_mask(lat,lon,tmaskutil) # get mask of Greenland region - needs hand editing for different models
    region = grl_mask(lat,lon,landseamask)  # get weighted mask of region to hose
    hflux = np.ma.masked_equal(tmaskutil,0.0)*0.0  # Create masked array
    hflux.data[:]=region[:]
    area_of_hosing = (dx*dy*region).sum() # calc area of hosing region
    hosing_per_unit_area = hosing_rate * 1.0e6 * 1000.0/area_of_hosing  # rate_in_Sv * 1.0e6 * rho_fw /area = kg/m2/s 
    hflux *= hosing_per_unit_area  # hosing flux in kg/m2/s
    # Create cube
    hflux_cube = iris.cube.Cube(hflux,  long_name='hosing', var_name='hosing', units='kg m-2 s-1')
    lat_coord = iris.coords.AuxCoord(lat,standard_name='latitude',units='degrees')
    lon_coord = iris.coords.AuxCoord(lon,standard_name='longitude',units='degrees')
    hflux_cube.add_aux_coord(lat_coord,[0,1])
    hflux_cube.add_aux_coord(lon_coord,[0,1])
    # Save as netcdf
    iris.save(hflux_cube,fileout, unlimited_dimensions=[])
    return hflux_cube
